import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:unicons/unicons.dart';

import 'package:random/cubit/game_cubit.dart';
import 'package:random/models/game_data.dart';
import 'package:random/ui/widgets/game_board.dart';

class GamePage extends StatefulWidget {
  const GamePage({super.key});

  @override
  State<GamePage> createState() => _GamePageState();
}

class _GamePageState extends State<GamePage> with TickerProviderStateMixin {
  static const boardSize = 6;

  List<List<Animation<double>?>> animations = List.generate(
    boardSize,
    (i) => List.generate(
      boardSize,
      (i) => null,
    ),
  );

  void resetAnimations() {
    animations = List.generate(
      boardSize,
      (i) => List.generate(
        boardSize,
        (i) => null,
      ),
    );
  }

  void createNewGame(GameCubit gameCubit) {
    final GameData newGame = GameData.createRandom(boardSize);
    gameCubit.updateGameState(newGame);
  }

  void removeCell(GameCubit gameCubit, int x, int y) {
    final GameData newGame = gameCubit.state.game ?? GameData.createRandom(boardSize);

    // "remove" cell
    newGame.updateCellValue(x, y, null);
    setState(() {});

    // "move down" cells
    final controller = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 750),
    )..addListener(() {
        if (mounted) {
          setState(() {});
        }
      });

    if (mounted) {
      setState(() {});
    }

    Animation<double> animation = Tween(
      begin: 0.0,
      end: 1.0,
    ).animate(CurvedAnimation(
      curve: Curves.bounceOut,
      parent: controller,
    ))
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          for (var i = 0; i < y; i++) {
            newGame.updateCellValue(x, (y - i), newGame.getCellValue(x, (y - i) - 1));
          }
          newGame.updateCellValue(x, 0, null);

          resetAnimations();
          setState(() {});

          controller.dispose();
        }
      });

    for (var i = 0; i < y; i++) {
      animations[(y - i) - 1][x] = animation;
    }

    controller.forward().orCancel;
  }

  void updateCellValue(GameCubit gameCubit, int x, int y, int value) {
    final GameData newGame = gameCubit.state.game ?? GameData.createRandom(boardSize);
    newGame.updateCellValue(x, y, value);

    gameCubit.updateGameState(newGame);
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (context, gameState) {
        const boardSize = 6;
        const double boardWidgetWidth = 300;
        const double boardWidgetHeight = 300;

        final GameCubit gameCubit = BlocProvider.of<GameCubit>(context);

        if (gameState.game?.isReady != true) {
          createNewGame(gameCubit);
        }

        return Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            GestureDetector(
              child: gameState.game != null
                  ? GameBoardWidget(
                      gameData: gameState.game!,
                      size: Size(boardWidgetWidth, boardWidgetHeight),
                      animations: animations,
                    )
                  : SizedBox.shrink(),
              onTapUp: (details) {
                double xTap = details.localPosition.dx;
                double yTap = details.localPosition.dy;

                int x = (xTap / boardWidgetWidth * boardSize).toInt();
                int y = (yTap / boardWidgetHeight * boardSize).toInt();
                print('[' + x.toString() + ',' + y.toString() + ']');

                removeCell(gameCubit, x, y);
              },
            ),
            IconButton(
              onPressed: () {
                createNewGame(gameCubit);
              },
              icon: Icon(UniconsSolid.star),
              color: Colors.white,
            ),
          ],
        );
      },
    );
  }
}
